"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
require("reflect-metadata");
/**
 * Method decorator factory. Marks method as not being usable in a web/JS/TS environment. Emits a
 * warning if method is called. Automatically adds it into a Reflect.defineMetadata compartment
 * marking web-unfriendly methods on the class, when containing class is instantiated.
 *
 * Intended use: when transpiling from another OO language, if interfaces are created that
 * the TS environment must satisfy, and these interfaces contain methods that don't make sense
 * in a JS/TS/web environment, then they should be wrapped with this method to prevent anyone
 * from calling them.
 *
 * @param {string} alternative? - Method that should be used instead of the one called.
 * @param {string} envUsage - Environment the method is intended for use in.
 *
 * @return {Function} Actual decorator, for wrapping methods (this is a decorator factory).
 */
function notForWebUse(alternative, envUsage) {
    if (envUsage === void 0) { envUsage = 'native mobile client or Java server'; }
    return function (target, propertyKey, descriptor) {
        Reflect.defineMetadata('nonWebMethods', target.name + " :: " + propertyKey, target, '${target.name}_${propertyKey}');
        var originalMethod = descriptor.value;
        descriptor.value = function () {
            var args = [];
            for (var _i = 0; _i < arguments.length; _i++) {
                args[_i] = arguments[_i];
            }
            console.warn("Method " + propertyKey + " on class " + target.constructor.name + " cannot be used in a " +
                ("Javascript/Typescript/web environment - it is for " + envUsage + " usage only. ") +
                (alternative ? 'Use ' + alternative + ' instead.' : ''));
            return originalMethod.apply(this, args);
        };
        return descriptor;
    };
}
exports.notForWebUse = notForWebUse;
exports.methodNotForWebUse = notForWebUse;
/**
 * Determine the decorator declaration type based on the arguments it receives.
 *
 *    Less than 1, or more than 3 arguments                                    --> INVALID
 *    1 arg  : arg1 is function                                                --> CLASS
 *    2 args : arg1 is function; arg2 is string or symbol                      --> STATIC_PROPERTY
 *    2 args : arg1.constructor is function, arg2 is string                    --> INSTANCE_PROPERTY
 *    3 args : arg3 is number                                                  --> PARAMETER
 *    3 args : arg1 is function w prototype & constructor; arg3 is descriptor  --> STATIC_METHOD
 *    3 args : arg1.constructor is function; arg3 is descriptor w no get, set  --> INSTANCE_METHOD
 *    3 args : arg1.constructor is function; arg3 is descriptor w get &/or set --> ACCESSOR
 *
 * @example Usage on class:
 *     @someDecorator
 *     class MyClass { ... }
 *     // On MyClass init, emits: "someDecorator declaration type: CLASS"
 *
 * @example Usage on parts of in a class:
 *     class MyClass {
 *         @someDecorator
 *         username: string = '';
 *         // On MyClass init, emits: "someDecorator declaration type: INSTANCE_PROPERTY"
 *
 *         @someDecorator
 *         fetchData() { ... }
 *         // On MyClass init, emits: "someDecorator declaration type: INSTANCE_METHOD"
 *     }
 *
 * @param {any[]} args Arguments initially passed to a function by decorator syntax e.g. @decorator
 *                     The "implicit" arguments given to a decorator by virtue of its placement.
 *
 * @return {string} CLASS, STATIC_PROPERTY, INSTANCE_PROPERTY, PARAMETER, ACCESSOR,
 *                  STATIC_METHOD, INSTANCE_METHOD, or INVALID (based on detected type).
 */
function getDecoratorType() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    var target = args[0], key = args[1], descriptor = args[2];
    var isTargetFunction = typeof target === 'function';
    var isTargetObject = typeof target === 'object';
    var isTargetConstructorFunction = typeof target.constructor === 'function';
    var targetHasPrototype = typeof target.prototype === 'object';
    var isKeyStringOrSymbol = typeof key === 'string' || typeof key === 'symbol';
    switch (args.filter(function (arg) { return typeof arg !== 'undefined' && arg !== null; }).length) {
        case 1:
            if (isTargetFunction)
                return 'CLASS';
            return 'INVALID';
        case 2:
            if (isTargetFunction && isKeyStringOrSymbol)
                return 'STATIC_PROPERTY';
            if (isTargetConstructorFunction && typeof key === 'string')
                return 'INSTANCE_PROPERTY';
            return 'INVALID';
        case 3:
            if (typeof descriptor === 'number')
                return 'PARAMETER';
            if (isTargetFunction && targetHasPrototype && isTargetConstructorFunction) {
                return 'STATIC_METHOD';
            }
            if (isTargetObject && isTargetConstructorFunction && typeof key === 'string') {
                return descriptor.get || descriptor.set ? 'ACCESSOR' : 'INSTANCE_METHOD';
            }
            return 'INVALID';
        default:
            return 'INVALID';
    }
}
exports.getDecoratorType = getDecoratorType;
/***************************************** BARREL EXPORTS *****************************************/
var types_iso_1 = require("./types-iso");
exports.singleton = types_iso_1.singleton;
//# sourceMappingURL=decorator.js.map