"use strict";
// TODO Test sortObject
// TODO Test inspectKeyTree
// TODO Consider returning isObject, and make it a proper "is"-typed function
Object.defineProperty(exports, "__esModule", { value: true });
/******************************************** IMPORTS *********************************************/
var types_iso_1 = require("./types-iso");
var array_1 = require("./array");
var deepFreezeStrict = require("deep-freeze-strict");
/********************************************* CONFIG *********************************************/
var assign = Object.assign, keys = Object.keys;
var braceMatchRegex = /(([^\[\]]+)|([[^\[\]]*\]))/g;
/********************************************* OBJECT *********************************************/
/**
 * Return a deep-frozen clone of a group of objects
 * Completely safe, doesn't freeze the original object
 *
 * @param {...Object[]} args Any # of objects to merge together into the merged clone object
 * @return {Object} Frozen merged version of provided objects; Clones originals - no mutation
 */
exports.assignFrozenClone = function () {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return deepFreezeStrict(assign.apply(void 0, [{}].concat(args)));
};
/**
 * [MUTATIVE] Deep freeze the given object
 *
 * @param {Object} obj Object to deeply freeze
 * @return {Readme<Object>} The original object, frozen
 *                          Note that it freezes the object itself - it doesn't
 *                          create a frozen copy (the return is for convenience)
 */
exports.deepFreeze = function (obj) { return deepFreezeStrict(obj); };
/**
 * Run the given function on the given object
 * Iterator operates on the value & key of any object provided, in order: 'val', 'key'
 *
 * @param {Function} to iterate over provided object -- (val, key) => void | any
 * @param {T extends object} obj Object to iterate over
 * @return {T extends Object} Returns the object initially passed in (for chaining)
 */
exports.eachPair = function (func) { return function (obj) {
    keys(obj).forEach(function (key) { return func(obj[key], key); });
    return obj;
}; };
/**
 * Get number of keys/pairs in an object
 * If given a non-object, return 0
 *
 * @param {Object} obj Object to get the number of keys of
 * @return {number} Number of keys in the object, or 0 if it's a non-object (or has no keys)
 */
exports.numKeys = function (obj) {
    if (typeof obj !== 'object' || obj == null || obj == undefined)
        return 0;
    return keys(obj).length;
};
exports.numPairs = exports.numKeys;
/**
 * Powerful key inspection tool
 * Shows keys of object and all objects in its prototype chain
 * Displays object name at each layer in the chain
 *
 * @param {Object} obj Object to get the keys of
 * @param {Object} showHidden If true, also display hidden keys.
 * @param {boolean} showProtoChainPosition If true, log objects showing each key's prototype
 *                                         chain position & the associated objects' names
 * @return {string[]} List of keys in obj & its prototype chain (w/ hidden keys if showHidden=true)
 */
exports.inspectKeyTree = function (obj, showHidden, showProtoChainPosition) {
    if (showHidden === void 0) { showHidden = true; }
    if (showProtoChainPosition === void 0) { showProtoChainPosition = false; }
    var getKeys = showHidden ? Object.getOwnPropertyNames : keys;
    var getName = function (obj) { return obj && obj.constructor && obj.constructor.name; };
    var proto = obj && obj.__proto__;
    var proto2 = proto && proto.__proto__;
    var proto3 = proto2 && proto2.__proto__;
    var proto4 = proto3 && proto3.__proto__;
    var proto5 = proto4 && proto4.__proto__;
    var proto6 = proto5 && proto5.__proto__;
    var proto7 = proto6 && proto6.__proto__;
    var proto8 = proto7 && proto7.__proto__;
    var objData = { name: getName(obj), keys: getKeys(obj) };
    if (proto)
        assign(objData, { __proto__: { name: getName(proto), keys: getKeys(proto) } });
    if (proto2)
        assign(objData, { __proto__2: { name: getName(proto2), keys: getKeys(proto2) } });
    if (proto3)
        assign(objData, { __proto__3: { name: getName(proto3), keys: getKeys(proto3) } });
    if (proto4)
        assign(objData, { __proto__4: { name: getName(proto4), keys: getKeys(proto4) } });
    if (proto5)
        assign(objData, { __proto__5: { name: getName(proto5), keys: getKeys(proto5) } });
    if (proto6)
        assign(objData, { __proto__6: { name: getName(proto6), keys: getKeys(proto6) } });
    if (proto7)
        assign(objData, { __proto__7: { name: getName(proto7), keys: getKeys(proto7) } });
    if (proto8)
        assign(objData, { __proto__8: { name: getName(proto8), keys: getKeys(proto8) } });
    if (showProtoChainPosition) {
        console.log(assign({}, objData, {
            prototypeKeys: obj.prototype ? getKeys(obj.prototype) : [],
            constructorKeys: obj.constructor ? getKeys(obj.constructor) : [],
        }));
    }
    var allKeysInPrototypeChain = getKeys(objData)
        .reduce(function (acc, collKey) {
        return collKey === 'name' || collKey === 'keys' ? acc : acc.concat(objData[collKey].keys);
    }, [])
        .concat(objData.keys || []);
    console.log(allKeysInPrototypeChain);
    return allKeysInPrototypeChain;
};
exports.inspectKeys = exports.inspectKeyTree;
exports.keyInspector = exports.inspectKeyTree;
exports.keyTreeInspector = exports.inspectKeyTree;
/**
 * Determine if an object contains a given key
 *
 * @param {Object} obj Object to check for the given key
 * @param {string} matchKey Key to search for in obj
 * @return {boolean} true if obj contains matchKey
 */
exports.hasKey = function (obj, matchKey) {
    if (typeof obj === 'object' && obj != null) {
        return keys(obj).some(function (k) { return k === matchKey; });
    }
    return false;
};
exports.containsKey = exports.hasKey;
exports.includesKey = exports.hasKey;
/*********************************** ADD NEW OBJECT PROPERTIES ************************************/
/**
 * Select correct defineProperty (for use with defineImmutableProp)
 */
var defineProperty = (Reflect && Reflect.defineProperty) || Object.defineProperty;
/**
 * Define a new property on an object without overwriting existing property
 *
 * @param {Object} obj Object being merged into
 * @param {string|number|symbol} keyName Name of property to assign value at
 * @param {RealAny} value Value to assign to property on object 'obj' (first param)
 * @param {boolean} mutable If true, make new property mutable [Default: false]
 * @return {Object} with new property added
 */
exports.defineProp = function (obj, keyName, value, mutable, enumerable) {
    if (mutable === void 0) { mutable = 'immutable'; }
    if (enumerable === void 0) { enumerable = true; }
    defineProperty(obj, keyName, {
        enumerable: enumerable,
        configurable: mutable === 'deletable',
        writable: mutable !== false && mutable !== 'immutable',
        value: value,
    });
    return obj;
};
exports.defineProperty = exports.defineProp;
/**
 * Define a new method on an object. Naturally immutable & non-enumerable
 * Must be a function
 *
 * @param {Object} obj Object being merged into
 * @param {string|number|symbol} keyName Name of property to add function at
 * @param {Function} value Function to assign to key on object 'obj' (1st param in NewKVPair)
 * @return {Object} with new property added
 */
exports.defineMethod = function (obj, keyName, func) {
    defineProperty(obj, keyName, {
        enumerable: false,
        configurable: false,
        writable: false,
        value: func,
    });
    return obj;
};
/**
 * Define an immutable public property on an object
 * Does not overwrite existing property
 *
 * @prop {Object} obj - Object being merged into.
 * @prop {string|number|symbol} keyName - Name of new prop to add to the gven object.
 * @prop {string} propVal - Actual value to assign to the new property.
 * @return {Object} Initial object with given property added
 */
exports.defineImmutableProp = function (obj, keyName, propVal) {
    var res = exports.defineProp(obj, keyName, propVal, 'immutable');
    return (res || obj);
};
exports.defineImmutableMethod = exports.defineImmutableProp;
exports.addImmutableProp = exports.defineImmutableProp;
exports.addImmutableMethod = exports.defineImmutableProp;
/**
 * Define a mutable but not deletable public property on an obj, without
 * overwriting existing props
 *
 * @prop {Object} obj Object being merged into
 * @prop {string|number|symbol} keyName Name of new prop to add to the gven object
 * @prop {string} propVal Actual value to assign to the new property
 * @return {Object} Initial object with given property added
 */
exports.defineMutableProp = function (obj, keyName, propVal) {
    exports.defineProp(obj, keyName, propVal, 'mutable');
    return obj;
};
exports.defineMutableMethod = exports.defineMutableProp;
exports.addMutableProp = exports.defineMutableProp;
exports.addMutableMethod = exports.defineMutableProp;
/**
 * Define a deletable & mutable public property on an object, without
 * overwriting existing props
 *
 * @prop {Object} obj Object being merged into
 * @prop {string|number|symbol} keyName Name of new prop to add to the gven object
 * @prop {string} propVal Actual value to assign to the new property
 * @return {Object} Initial object with given property added
 */
exports.defineDeletableProp = function (obj, keyName, propVal) {
    exports.defineProp(obj, keyName, propVal, 'deletable');
    return obj;
};
exports.defineDeletableMethod = exports.defineDeletableProp;
exports.addDeletableProp = exports.defineDeletableProp;
exports.addDeletableMethod = exports.defineDeletableProp;
/**
 * Define a public mutable (even deletable) getter property on an object
 * @generic <R> Return object
 *
 * @prop {Object} obj Object being merged into
 * @prop {string|number|symbol} keyName Name of new getter prop to add to the gven object
 * @prop {string} propVal Actual value to assign to the new getter property
 *
 * @return {Object} Initial object with given property added
 */
exports.defineGetterProp = function (obj, keyName, propVal) {
    defineProperty(obj, keyName, { enumerable: true, configurable: true, get: propVal });
    return obj;
};
exports.addGetterProp = exports.defineGetterProp;
exports.addGetter = exports.defineGetterProp;
exports.defineGetter = exports.defineGetterProp;
exports.addGetProp = exports.defineGetterProp;
exports.defineGetProp = exports.defineGetterProp;
/******************************************** SORTING *********************************************/
/**
 * Sort an object by its keys, return duplicate object
 *
 * @param {Object} obj Source object (sort its keys)
 * @return {Object} Duplicate of input object with keys sorted
 */
exports.sortObject = function (obj) {
    return keys(obj)
        .sort()
        .reduce(function (acc, key) {
        var _a;
        return assign(acc, (_a = {}, _a[key] = obj[key], _a));
    }, {});
};
/********************************************** OMIT **********************************************/
function isNonArrayOrFuncObj(val) {
    return val != null && typeof val === 'object' && Array.isArray(val) === false;
}
function isObjectObject(val) {
    return isNonArrayOrFuncObj(val) && Object.prototype.toString.call(val) === '[object Object]';
}
function isPlainObject(val) {
    // If it's an object that returns [object Object]'
    if (isObjectObject(val) === false)
        return false;
    // If it has a modified constructor
    if (typeof val.constructor !== 'function')
        return false;
    // If it has a modified prototype
    if (isObjectObject(val.constructor.prototype) === false)
        return false;
    // If constructor does not have an Object-specific method
    if (val.constructor.prototype.hasOwnProperty('isPrototypeOf') === false)
        return false;
    // Most likely a plain Object
    return true;
}
function isObject(val) {
    return val != null && (isPlainObject(val) || typeof val === 'function' || Array.isArray(val));
}
/*
 * ACTUAL OMIT IMPLEMENTATION
 */
function omit(obj, props, fn) {
    if (!isObject(obj))
        return {};
    if (typeof props === 'function') {
        fn = props;
        props = [];
    }
    if (typeof props === 'string')
        props = [props];
    return keys(obj).reduce(function (acc, k) {
        var _a;
        return !props ||
            (props.indexOf(k) === -1 && (!types_iso_1.isFunction(fn) || fn(obj[k], k, obj)))
            ? assign(acc, (_a = {}, _a[k] = obj[k], _a))
            : acc;
    }, {});
}
exports.omit = omit;
/************************************** SAFE PROPERTY ACCESS **************************************/
/**
 * Safely get the given prop (via array of path props or 'access string') from
 * the given object
 *
 * @param {string[]|string|number} propPath String in 'key1.key2.etc' form, number,
 *                                 or array of strings where each item is a key
 *                                 to traverse into:
 *                                 e.g.: ['key1', 'key2', 'etc'] refers to key1.key2.etc
 * @param {Object} obj Object to get the value from using the given path
 * @return {any} Value found at the given path
 */
exports.get = function (obj, propPath, defaultValue) {
    if (defaultValue === void 0) { defaultValue = undefined; }
    // Handle obj value of null
    if (obj === null)
        return obj;
    // Handle undefined obj
    if (typeof obj === "undefined")
        return defaultValue;
    // Handle bad or empty prop paths
    if (propPath === "" || propPath == null || typeof propPath === "undefined")
        return defaultValue;
    // Handle numeric prop paths
    if (typeof propPath === "number")
        return obj[propPath.toString()];
    // Parse property path
    var propArr = typeof propPath === "string"
        ? array_1.rmAllFalsy(array_1.flatten(propPath
            .replace(/\.\.+/g, ".")
            .split(".")
            .map(function (str) {
            var match = str.match(braceMatchRegex);
            return match && match.filter(function (s) { return s !== "]" && s !== "["; });
        })))
        : propPath;
    // Walk obj by property path array
    return propArr.reduce(function (acc, key) {
        if (typeof acc === "undefined" || acc === null)
            return defaultValue;
        if (typeof acc[key] === "undefined")
            return defaultValue;
        return acc[key];
    }, obj);
};
//# sourceMappingURL=object.js.map